// *******************************************************
// * "Color picker" web application JavaScript functions *
// *******************************************************

var colGroups = new Array("pink", "purple", "red", "orange", "yellow", "green", "cyan", "blue", "brown", "white", "gray", "custom");
var groupCols = new Array(6, 18, 9, 5, 11, 22, 7, 16, 18, 17, 10);
var hexDigits = new Array( "0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "A", "B", "C", "D", "E", "F");

//
// 2-digits hexadecimal to decimal converter
//

function hex2dec(hex) {

    var dec = "null";

    if (hex.length == 2) {
        if (hexDigits.indexOf(hex.charAt(0)) != -1 && hexDigits.indexOf(hex.charAt(1)) != -1) {
            dec = 16 * hexDigits.indexOf(hex.charAt(0)) + hexDigits.indexOf(hex.charAt(1));
        }
    }
    return dec;

}

//
// Decimal to 2-digits hexadecimal converter
//

function dec2hex(dec) {

     var hex = "null";

     if (dec >= 0 && dec <= 255) {
         var dec2 = dec % 16;
         var dec1 = (dec - dec2) / 16;
         hex = hexDigits[dec1] + hexDigits[dec2];
     }
     return hex;

}

//
// HTML color code to RGB color codes converter
//

function html2rgb(htmlColor) {

    var rgbColor = "null";

    if (htmlColor.length == 7) {
        if (htmlColor.substr(0, 1) == '#') {
            var r = hex2dec(htmlColor.substr(1, 2));
            var g = hex2dec(htmlColor.substr(3, 2));
            var b = hex2dec(htmlColor.substr(5, 2));
            if (r != "null" && g != "null" && b != "null") {
                rgbColor = r + ', ' + g + ', ' + b;
            }
        }
    }
    return rgbColor;

}

//
// RGB color codes to HTML color code converter
//

function rgb2html(Red, Green, Blue) {

    var htmlColor = "null";

    var r = dec2hex(Red);
    var g = dec2hex(Green);
    var b = dec2hex(Blue);
    if (r != "null" && g != "null" && b != "null") {
        htmlColor = '#' + r + g + b;
    }
    return htmlColor;

}

//
// Color group selection: Display group color list (and make other side elements visible)
//

function colGroupSel(colGroup) {

    var i;

    var re = /^([a-z]*)$/i;
    if (re.test(colGroup)) {                                                   // check if colGroup is all letters
        if (document.getElementById) {
            // Hide all color lists
            for (i = 0; i < colGroups.length; i++) {
                document.getElementById(colGroups[i]).style.visibility = "collapse";
            }
            // Make text/background usage radiobuttons visible
            document.getElementById("rbusage").style.visibility = "visible";
            // Make selected group color list visible
            document.getElementById(colGroup).style.visibility = "visible";
            // Make color values tables visible
            document.getElementById("colvals").style.visibility = "visible";
            // Make colors preview elements visible
            document.getElementById("colprev").style.visibility = "visible";
        }
    }
}

//
// Color selection: Get color codes and call function to display color values and selected-colored text on selected-colored background preview
//

function colorSel(color) {

    var re = /^([a-z0-9_]*)$/i;
    if (re.test(color)) {                                                      // check if color is all letters, numbers or underscore
        if (document.getElementById) {
            // Get HTML color name and code from web side (based on argument passed)
            var htmlNameId = color + "_name";
            var htmlColId = color + "_hex";
            var htmlName = document.getElementById(htmlNameId).firstChild.data;
            var htmlCol = document.getElementById(htmlColId).firstChild.data;
            // Determine RGB color values
            var rgb = html2rgb(htmlCol);
            // Display the whole...
            display(htmlName, htmlCol, rgb);
        }
    }

}

//
// Custom RGB selection: Determine color codes and call function to display color values and selected-colored text on selected-colored background preview
//

function rgbSel() {

    var htmlCol, rgb; var i, j;
    var htmlName = '-----';

    if (document.getElementById) {
        if (document.getElementById("colcode_0").checked) {
            // RGB color codes are given in decimal format
            var decRed = document.getElementById("colred").value;
            var decGreen = document.getElementById("colgreen").value;
            var decBlue = document.getElementById("colblue").value;
            var re = /^([0-9]*)$/i;
            if (re.test(decRed) && re.test(decGreen) && re.test(decBlue)) {
                rgb = decRed + ', ' + decGreen + ', ' + decBlue;
                // Determine HTML color code from RGB values
                htmlCol = rgb2html(decRed, decGreen, decBlue);
            }
            else {
                htmlCol = "null";
            }
            if (htmlCol == "null") {
                alert('Invalid decimal RGB color values!');
            }
        }
        else {
            // RGB color codes are given in hexadecimal format
            var hexRed = document.getElementById("colred").value;
            var hexGreen = document.getElementById("colgreen").value;
            var hexBlue = document.getElementById("colblue").value;
            var re = /^([0-9A-F]*)$/i;
            if (re.test(hexRed) && re.test(hexGreen) && re.test(hexBlue)) {
                htmlCol = '#' + hexRed + hexGreen + hexBlue;
                // Determine RGB color code from HTML color code
                rgb = html2rgb(htmlCol);
            }
            else {
                rgb = "null";
            }
            if (rgb == "null") {
                alert('Invalid hexadecimal RGB color values!');
            }
        }
        // Try to find HTML color name (in webside tables)
        if (htmlCol != "null" && rgb != "null") {
            // Search all color tables until the actual color code has been found (or is not listed anwhere)
            for (i = 0; i < groupCols.length; i++) {
                for (j = 1; j <= groupCols[i]; j++) {
                    var htmlColId = colGroups[i] + j + "_hex";
                    if (htmlCol == document.getElementById(htmlColId).firstChild.data) {
                        var htmlNameId = colGroups[i] + j + "_name";
                        htmlName = document.getElementById(htmlNameId).firstChild.data;
                        break;
                    }
                }
                // Quit the loop, if the color name has been found
                if (htmlName != '-----') {
                    break;
                }
            }
            // Display the whole...
            display(htmlName, htmlCol, rgb);
        }
    }

}

// Display color values tables, ccolored preview and corrsponding CSS style rule

function display(htmlName, htmlCol, rgb) {

    // Free Pascal TColor codes (Free Pascal coding) and names
    var fpColors = new Array(
        "$000000", "$000080", "$008000", "$008080", "$800000", "$800080", "$808000", "$808080", "$C0C0C0", "$0000FF",
        "$00FF00", "$00FFFF", "$FF0000", "$FF00FF", "$FFFF00", "$FFFFFF", "$F0FBFF", "$A4A0A0", "$C0DCC0", "$F0CAA6"
    );
    var fpColorNames = new Array(
        "clBlack", "clMaroon", "clGreen", "clOlive", "clNavy", "clPurple", "clTeal", "clGray (clDkGray)", "clSilver (clLtGray)", "clRed",
        "clLime", "clYellow", "clBlue", "clFuchsia", "clAqua", "clWhite", "clCream", "clMedGray", "clMoneyGreen", "clSkyBlue"
    );
    // A selection of the Perl "Chart" color values (decimal RGB) and names
    var chartColors = new Array(
        "255, 255, 255", "0, 0, 0", "200, 0, 0", "0, 175, 0", "0, 0, 200", "250, 125, 0", "225, 225, 0", "200, 0, 200",
        "0, 125, 250", "125, 250, 0", "145, 0, 250", "250, 0, 125", "250, 125, 125", "125, 125, 0", "125, 0, 125", "0, 125, 125",
        "200, 125, 125", "160, 80, 0", "225, 225, 225", "255, 105, 180", "0, 0, 139", "138, 43, 226", "255, 218, 185", "144, 238, 144",
        "230, 230, 250", "147, 112, 219", "255, 127, 0", "106, 90, 205", "138, 43, 226", "65, 105, 225", "250, 235, 215", "95, 158, 160",
        "184, 134, 11", "85, 107, 47", "153, 50, 204", "143, 188, 143", "47, 79, 79", "255, 20, 147", "0, 191, 255", "30, 144, 255",
        "205, 92, 92", "255, 240, 245", "255, 250, 205", "173, 216, 230", "224, 255, 255", "238, 221, 130", "255, 182, 193",
        "255, 160, 122", "135, 206, 250", "176, 196, 222", "255, 255, 224", "186, 85, 211", "255, 228, 225", "255, 222, 173",
        "107, 142, 35", "255, 69, 0", "152, 251, 152", "175, 238, 238", "219, 112, 147", "188, 143, 143", "46, 139, 87", "135, 206, 235",
        "112, 128, 144", "0, 255, 127", "70, 130, 180", "208, 32, 144", "127, 255, 212", "240, 255, 255", "255, 228, 196", "0, 0, 255",
        "222, 184, 135", "127, 255, 0", "210, 105, 30", "255, 127, 80", "255, 248, 220", "0, 255, 255", "178, 34, 34", "255, 215, 0",
        "218, 165, 32", "190, 190, 190", "0, 255, 0", "240, 255, 240", "255, 255, 240", "240, 230, 140", "255, 0, 255", "176, 48, 96",
        "255, 165, 0", "255, 0, 0", "250, 128, 114", "255, 245, 238", "160, 82, 45", "255, 250, 250", "210, 180, 140", "216, 191, 216",
        "255, 99, 71", "245, 222, 179", "255, 255, 0", "218, 112, 214", "139, 0, 139", "139, 0, 0", "154, 205, 50", "102, 205, 170",
        "0, 139, 139", "0, 0, 205", "205, 133, 63", "139, 69, 19"
    );
    var chartColorNames = new Array(
        "white", "black", "red", "green", "blue", "orange", "yellow", "purple", "light_blue", "light_green",
        "light_purple", "pink", "peach", "olive", "plum", "turquoise", "mauve", "brown", "grey", "HotPink", "DarkBlue",
        "BlueViolet", "PeachPuff", "LightGreen", "lavender", "MediumPurple", "DarkOrange", "SlateBlue", "BlueViolet",
        "RoyalBlue", "AntiqueWhite", "CadetBlue", "DarkGoldenrod", "DarkOliveGreen", "DarkOrchid", "DarkSeaGreen", "DarkSlateGray",
        "DeepPink", "DeepSkyBlue", "DodgerBlue", "IndianRed", "LavenderBlush", "LemonChiffon", "LightBlue", "LightCyan",
        "LightGoldenrod", "LightPink", "LightSalmon", "LightSkyBlue", "LightSteelBlue", "LightYellow", "MediumOrchid", "MistyRose",
        "NavajoWhite", "OliveDrab", "OrangeRed", "PaleGreen", "PaleTurquoise", "PaleVioletRed", "RosyBrown", "SeaGreen", "SkyBlue",
        "SlateGray", "SpringGreen", "SteelBlue", "VioletRed", "aquamarine", "azure", "bisque", "blue1", "burlywood", "chartreuse",
        "chocolate", "coral", "cornsilk", "cyan", "firebrick", "gold", "goldenrod", "gray", "green1", "honeydew", "ivory", "khaki",
        "magenta", "maroon", "orange1", "red1", "salmon", "seashell", "sienna", "snow", "tan", "thistle", "tomato", "wheat", "yellow1",
        "orchid", "magenta4", "red4", "OliveDrab3", "aquamarine3", "cyan4", "blue3", "tan3", "chocolate4"
    );

    if (document.getElementById) {
        // Determine Free Pascal TColor value
        var fpCol = '$' + htmlCol.substr(5, 2) + htmlCol.substr(3, 2) + htmlCol.substr(1, 2);
        // Get Free Pascal TColor name (if it exists)
        var fpName = '-----';
        if (fpColors.indexOf(fpCol) != -1) {
            fpName = fpColorNames[fpColors.indexOf(fpCol)];
        }
        // Get Perl "Chart" color name (if it exists in table)
        var chartName = ''; var i = 0;
        if (chartColors.indexOf(rgb) != -1) {
            chartName = chartColorNames[chartColors.indexOf(rgb)];
        }
        // As I include the Perl "Chart" color names for all HTML colors in the different lists, not finding a name means that it does not exist
        if (chartName == '' && htmlName != '-----') {
            chartName = '-----';
        }
        // Fill in the color values table (text or background color, depending on actual selection)
        if (document.getElementById("usage_0").checked) {
            // Text color values
            document.getElementById("fg_htmlname").firstChild.data = htmlName;
            document.getElementById("fg_htmlcol").firstChild.data = htmlCol;
            document.getElementById("fg_rgb").firstChild.data = rgb;
            document.getElementById("fg_fpcol").firstChild.data = fpCol;
            document.getElementById("fg_fpname").firstChild.data = fpName;
            document.getElementById("fg_chartname").firstChild.data = chartName;
        }
        else {
            // Background color values
            document.getElementById("bg_htmlname").firstChild.data = htmlName;
            document.getElementById("bg_htmlcol").firstChild.data = htmlCol;
            document.getElementById("bg_rgb").firstChild.data = rgb;
            document.getElementById("bg_fpcol").firstChild.data = fpCol;
            document.getElementById("bg_fpname").firstChild.data = fpName;
            document.getElementById("bg_chartname").firstChild.data = chartName;
        }
        // Show color preview
        var textcol = ''; var bgcol = '';
        if (document.getElementById("usefg").checked) {
            // If text color is selected to be used, get it from color values table
            if (document.getElementById("fg_htmlname").firstChild.data) {
                // Get the color value as color name
                textcol = document.getElementById("fg_htmlname").firstChild.data;
                // Delete the alternate names for "Magenta" and "Cyan"
                textcol = textcol.replace(' (Fuchsia)', '');
                textcol = textcol.replace(' (Aqua)', '');
                // If no color name has been found, use the HTML color code instead
                if (textcol == '-----') {
                    textcol = document.getElementById("fg_htmlcol").firstChild.data;
                }
            }
        }
        else {
            // If text color is selected not to be used, use "Black"
            textcol = "Black";
        }
        if (document.getElementById("usebg").checked) {
            // If background color is selected to be used, get it from color values table
            if (document.getElementById("bg_htmlname").firstChild.data) {
                // Get the color value as color name
                bgcol = document.getElementById("bg_htmlname").firstChild.data;
                // Delete the alternate names for "Magenta" and "Cyan"
                bgcol = bgcol.replace(' (Fuchsia)', '');
                bgcol = bgcol.replace(' (Aqua)', '');
                // If no color name has been found, use the HTML color code instead
                if (bgcol == '-----') {
                    bgcol = document.getElementById("bg_htmlcol").firstChild.data;
                }
            }
        }
        else {
            // If background color is selected not to be used, use "White"
            bgcol = "White";
        }
        // Construct the CSS style rule
        var style = '';
        if (textcol.length > 1 || bgcol.length > 1) {                       // this avoids the display at application startup
            style = 'text-align:center; ';                                  // text will always be centered
            if (textcol.length > 1) {
                style += 'color:' + textcol + '; ';                         // text color (if any defined)
            }
            if (bgcol.length > 1) {
                style += 'background:' + bgcol + '; ';                      // background color (if any defined)
            }
            if (document.getElementById("large").checked) {                 // normal or large font, depending on user selection
                style += 'font-size:150%; ';
            }
            style += 'font-weight:bold';                                    // text will always be bold
        }
        // If there is a style rule, apply it to the colored preview and display it (as text)
        if (style.length > 1) {
            // Apply CSS style to preview
            document.getElementById("preview").style = style;
            // Use the text entered by the user
            document.getElementById("preview").firstChild.data = document.getElementById("text").value;
            // Make the elements concerning the display of the CSS style rule visible
            document.getElementById("tstyle").style.visibility = "visible";
            // Display the CSS style rule
            document.getElementById("style").firstChild.data = 'style="' + style + '"';
        }
    }

}
